@@ -87,7 +87,7 @@ class StrollerController(TGController):
         if force_user and bool(predicates.in_group('stroller')):
             user = DBSession.query(User).filter_by(user_id=force_user).first()
 
-        orders = Order.find_by_data('user_id', user.user_id).order_by(Order.uid.desc()).all()
+        orders = Order.find_by_data('user_id', user.user_id).filter(Order.confirmed==True).order_by(Order.uid.desc()).all()
         if not uid and orders:
             uid = orders[0].uid
         return dict(user=user, orders=orders, selected_order=uid)
