@@ -4,11 +4,12 @@ from tw.forms import TableForm, SingleSelectField, FormField, TextField, Multipl
 from tw.dynforms import OtherSingleSelectField, HidingContainerMixin, HidingSingleSelectField
 from formencode import validators
 
-from tg import expose, flash, require, url, request, redirect, session, TGController, validate
+from tg import expose, flash, require, url, request, redirect, session, TGController, validate, config
 from pylons.i18n import ugettext as _, lazy_ugettext as l_
 from repoze.what import predicates
 from webob.exc import HTTPNotFound
 
+from stroller.lib import stroller_url, style_css, send_email
 from stroller.lib import stroller_url, manage_css, language
 from stroller.model.commerce import *
 from stroller.model.core import DBSession, User, Group
@@ -27,7 +28,7 @@ __all__ = ['ManageController']
 class CreateUserForm(TableForm):
         class fields(WidgetsList):
             insert_username = TextField(validator=validators.String(not_empty=True, strip=True))
-            insert_email = TextField(validator=validators.Email(not_empty=True))
+            insert_email = TextField(validator=validators.Email(not_empty=True, strip=True))
 createuser_form = CreateUserForm("createuser_form")
 
 class AddUserForm(TableForm):
@@ -103,13 +104,15 @@ class PhotoForm(twf.TableForm):
         photo = twf.FileField(label_text="Photo", validator=validators.FieldStorageUploadConverter())
 add_photo_form = PhotoForm(submit_text="Upload")        
 
-def thumbnail(what, size=(128,128)):
+def thumbnail(what, size=None):
+    if not size:
+        size = map(lambda x: int(x.strip()), config.get('stroller_thumbnail_size', '128, 128').split(','))
     thumb_data = StringIO()
 
     thumb = Image.open(what)
-    thumb.thumbnail(size)
+    thumb.thumbnail(size, Image.BILINEAR)
     thumb = thumb.convert('RGB')
-    thumb.save(thumb_data, 'JPEG')
+    thumb.save(thumb_data, 'JPEG', quality=85)
 
     return thumb_data.getvalue()
 
@@ -361,16 +364,23 @@ class ManageController(TGController):
 
         return redirect(dest)
          
+
+    @expose('stroller.templates.manage.orderedproducts')
+    def ordered_products(self, p):
+        p = Product.get(p)
+        return dict(product=p) if p else dict()
+
     @expose('stroller.templates.manage.orderlist')
     def list_orders(self):
-        unpaid_orders     = DBSession.query(Order).filter_by(payed=0).order_by(Order.uid)
+        Order.clear_unconfirmed()
+
+        unpaid_orders     = DBSession.query(Order).filter_by(payed=0).filter_by(confirmed=1).order_by(Order.uid)
         paid_orders       = DBSession.query(Order).filter_by(payed=1).filter_by(dispatched=0).order_by(Order.uid)
         dispatched_orders = DBSession.query(Order).filter_by(dispatched=1).order_by(Order.uid)
         return dict(unpaid_orders=unpaid_orders, paid_orders=paid_orders, dispatched_orders=dispatched_orders)
 
     @expose()
     def promote_order(self, uid):
-        
         order = DBSession.query(Order).filter_by(uid=uid).first()
         if not order.payed:
             if not order.get_data('ship_name') or order.get_data('ship_name').value == '':
@@ -379,15 +389,15 @@ class ManageController(TGController):
                 order.payed = True
         else:
             for p in order.items:
-                if p.product.stock and p.product.stock > -1:
-                    if p.product.stock < p.quantity:
-                        flash("Didn't - Stock doesn't have enough quantity of item "+p.product.name)
-                        return redirect(stroller_url('/manage/list_orders'))
+                if not p.product.in_stock(p.quantity):
+                    flash('Not enough {0} in stock'.format(p.product.name))
+                    return redirect(stroller_url('/manage/list_orders'))
+
             order.dispatched = True
+
             for p in order.items:
-                if p.product.stock > -1:
-                    if p.product.stock > p.quantity:
-                        p.product.stock -= p.quantity
+                p.product.remove_from_stock(p.quantity)
+
         return redirect(stroller_url('/manage/list_orders'))
 
     @expose()
@@ -395,8 +405,7 @@ class ManageController(TGController):
         order = DBSession.query(Order).filter_by(uid=uid).first()
         if order.dispatched:
             for p in order.items:
-                if p.product.stock and p.product.stock > -1:
-                    p.product.stock += p.quantity
+                p.product.revert_to_stock(p.quantity)
             order.dispatched = False
         else:
             order.payed = False
@@ -407,6 +416,13 @@ class ManageController(TGController):
         order = DBSession.query(Order).filter_by(uid=uid).first()
         return dict(order=order)
 
+    @expose()
+    def unconfirm_order(self, uid):
+        order = DBSession.query(Order).filter_by(uid=uid).first()
+        if order:
+            order.confirmed = False
+        flash(_('Order canceled'))
+        return redirect(stroller_url('/manage/list_orders'))
     
     @expose('stroller.templates.manage.orderedit')
     def edit_order(self, uid, **kw):
@@ -441,7 +457,7 @@ class ManageController(TGController):
         my_query = DBSession.query(Category).order_by(Category.name)
         my_options = [x.name for x in my_query]
         users = []
-        for user in DBSession.query(User):
+        for user in DBSession.query(User).order_by(User.user_id.desc()):
             user_groups = [g.group_name for g in user.groups]
             user_categories = [c.name for c in my_query if "stroller_private_%s" % c.uid in user_groups]
             user_child_args = dict(select_categories=dict(options = my_options))
@@ -470,6 +486,20 @@ class ManageController(TGController):
                        display_name=insert_username, password=insert_username)
         stroller_group.users.append(newuser)
         DBSession.add(newuser)
+        stroller_order_mail = config.get('stroller_order_notify_mail', None)
+        stroller_email_sender = config.get('stroller_email_sender', stroller_order_mail)
+        send_email(stroller_email_sender, insert_email, 'Account Registration', '''
+Your subscription was accepted.
+
+Please log in on the website with this personal credential to order the chosen items.
+
+Here are your account details:
+Username: {0}
+Password: {0}
+
+---
+Support Team
+'''.format(insert_username))
         flash('User successfully created')
         return redirect(stroller_url('/manage/usermanage'))
 
@@ -480,7 +510,10 @@ class ManageController(TGController):
             add_to_group = [add_to_group]
         
         user = DBSession.query(User).filter_by(user_id=kw['my_userid']).first()
-        
+        if not user:
+            flash('User not found')
+            return redirect(stroller_url('/manage/usermanage'))
+ 
         groups_to_delete = []
         for group in user.groups:
             if group.group_name.startswith('stroller_private') and \
